#include "Initiator.h"

// constructor
Initiator::Initiator( sc_module_name name )
    : sc_module( name )
    , i_socket( "i_socket" )
{
    i_socket.bind( *this );  // I/FȥХ

    SC_THREAD( thread0 );
}

void Initiator::write( unsigned int addr, unsigned int data, sc_time time )
{
    tlm::tlm_generic_payload* trans = new tlm::tlm_generic_payload;

    trans->set_address( addr );
    trans->set_data_ptr( reinterpret_cast<unsigned char*>(&data) );
    trans->set_data_length( 4 );
    trans->set_write();

    i_socket->b_transport( *trans, time );

    if (trans->is_response_error()) {
        cout << "Initiator::Write Access ERROR";
        cout << " :  Write Address=0x" << hex << addr;
        cout << " :  Write Data=0x" << hex << data << endl;
    }

    delete trans;
}

void Initiator::read( unsigned int addr, unsigned int& data, sc_time time )
{
    tlm::tlm_generic_payload* trans = new tlm::tlm_generic_payload;

    trans->set_address( addr );
    trans->set_data_ptr( reinterpret_cast<unsigned char*>(&data) );
    trans->set_data_length( 4 );
    trans->set_read();

    i_socket->b_transport( *trans, time );

    if (trans->is_response_error()) {
        cout << "Initiator::Read Access ERROR";
        cout << " :  Read Address=0x" << hex << addr << endl;
    }

    delete trans;
}

void Initiator::thread0()
{
    cout << "////////////////////// Simulation Start //////////////////////" << endl;

    unsigned int addr[3] = { 0x010, 0x020, 0x030 };
    unsigned int w_data[3] = { 0x1111, 0x2222, 0x3333 };
    unsigned int r_data[3];

    // write
    for (int i=0; i<3; i++) {
        write( addr[i], w_data[i] );

        cout << "Initiator::Write Access " ;
        cout << " :  Address=0x" << hex << addr[i];
        cout << " :  Data=0x" << hex << w_data[i] << endl;
    }

    // read
    for (int i=0; i<3; i++) {
        read( addr[i], r_data[i] );

        cout << "Initiator::Read Access " ;
        cout << " :  Address=0x" << hex << addr[i];
        cout << " :  Data=0x" << hex << r_data[i] << endl;
    }


    cout << "////////////////////// Simulation End ////////////////////////" << endl;
    sc_stop();
}

tlm::tlm_sync_enum Initiator::nb_transport_bw( tlm::tlm_generic_payload& trans, tlm::tlm_phase &phase, sc_time& time )
{
    // not support
    trans.set_response_status( tlm::TLM_GENERIC_ERROR_RESPONSE );
    return tlm::TLM_COMPLETED;
}

void Initiator::invalidate_direct_mem_ptr( sc_dt::uint64 a, sc_dt::uint64 b)
{
    // DMI unused
}

